package com.examples.testapp1;

import com.backendless.Backendless;
import com.backendless.persistence.DataQueryBuilder;
import com.examples.testapp1.data.Address;
import com.examples.testapp1.data.Auto;
import com.examples.testapp1.data.MaterialType;
import com.examples.testapp1.data.MedicalInfo;
import com.examples.testapp1.data.Person;
import com.examples.testapp1.data.Phone;
import com.examples.testapp1.data.ServiceOrder;
import com.examples.testapp1.data.WorkType;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Stream;

public class CreateRecordTest
{
  public static final String APPLICATION_ID = "A21CEE44-DF5F-2B1B-FF68-4FF44A62F700";
  public static final String API_KEY = "57178E28-C5B2-6C23-FFA7-51FE50964300";

  private static void onStart()
  {
    Backendless.Persistence.mapTableToClass( "person", Person.class );
    Backendless.Persistence.mapTableToClass( "address", Address.class );
    Backendless.Persistence.mapTableToClass( "phone", Phone.class );
    Backendless.Persistence.mapTableToClass( "medical_info", MedicalInfo.class );

    Backendless.Persistence.mapTableToClass( "auto", Auto.class );
    Backendless.Persistence.mapTableToClass( "material_type", MaterialType.class );
    Backendless.Persistence.mapTableToClass( "work_type", WorkType.class );
    Backendless.Persistence.mapTableToClass( "service_order", ServiceOrder.class );

    Backendless.initApp( APPLICATION_ID, API_KEY );
  }

  public static void main( String[] args )
  {
    onStart();
//    testPersonSaving();
    testServiceOrder();
  }

  public static void testPersonSaving()
  {
    Person person = new Person();
    person.setName( "John" );
    person.setName( "Malkovich" );
    person.setGender( "male" );
    person.setBirthday( Date.from( LocalDate.of( 1953, 12, 9 ).atStartOfDay( ZoneId.systemDefault() ).toInstant() ) );


    Address address1 = new Address();
    address1.setCity( "New York" );
    address1.setStreet( "North Avenue" );
    address1.setBuilding("07076");
    address1.setApartment( "2A" );

    Address address2 = new Address();
    address2.setCity( "Florida" );
    address1.setStreet( "South Avenue" );
    address1.setBuilding("34-789");
    address1.setApartment( "321" );

    ArrayList<Address> addresses = new ArrayList<>();
    addresses.add( address1 );
    addresses.add( address2 );


    Phone phone1 = new Phone();
    phone1.setNumber( "(111)876-345-799" );
    phone1.setType( "mobile" );

    Phone phone2 = new Phone();
    phone2.setNumber( "(222)345-786-54" );
    phone2.setType( "landline" );

    ArrayList<Phone> phones = new ArrayList<>();
    phones.add( phone1 );
    phones.add( phone2 );


    MedicalInfo medicalInfo = new MedicalInfo();
    medicalInfo.setBlood_type( "A+" );
    medicalInfo.setAllergens( "anice" );


    person.setAddress( addresses );
    person.setPhone( phones );
    person.setMedical_info( medicalInfo );


    String personId = Backendless.CustomService.invoke( "SaveObjectModel", "saveServiceOrder", new Object[] {person} );
    Person savedPerson = Backendless.Data.of( Person.class ).findById( personId );

    System.out.println("Person objectId: " + savedPerson.getObjectId());
  }



  public static enum Material
  {
    oil("oil"), brake_shoes("brake shoes");

    private String nameInDB;
    private String objectId;

    Material( String nameInDB )
    {
      this.nameInDB = nameInDB;
    }

    public String getObjectId()
    {
      return objectId;
    }

    public void setObjectId( String objectId )
    {
      this.objectId = objectId;
    }
  }

  public static enum Work
  {
    brake_shoes_replacement("replacement of brake shoes"), oil_change("oil change"), light_calibration("light calibration");

    private String nameInDB;
    private String objectId;

    Work( String nameInDB )
    {
      this.nameInDB = nameInDB;
    }

    public String getObjectId()
    {
      return objectId;
    }

    public void setObjectId( String objectId )
    {
      this.objectId = objectId;
    }
  }

  public static void getConstantData()
  {
    Stream.of( Material.values() ).forEach( material -> {
      DataQueryBuilder dataQueryBuilder = DataQueryBuilder.create().setWhereClause( "name='" + material.nameInDB + "'" );
      MaterialType materialType = Backendless.Data.of( MaterialType.class ).find( dataQueryBuilder ).get( 0 );
      material.setObjectId( materialType.getObjectId() );
    } );

    Stream.of( Work.values() ).forEach( work -> {
      DataQueryBuilder dataQueryBuilder = DataQueryBuilder.create().setWhereClause( "name='" + work.nameInDB + "'" );
      WorkType workType = Backendless.Data.of( WorkType.class ).find( dataQueryBuilder ).get( 0 );
      work.setObjectId( workType.getObjectId() );
    } );
  }

  public static void testServiceOrder()
  {
    onStart();
    getConstantData();

    // set to the customer personal discount
    Person person = Backendless.Data.of( Person.class ).findById( "19386945-F71B-162A-FFB4-0EE33D4FF300" );
    person.setDiscount( 0.05 );
    Backendless.Data.of( Person.class ).save( person );


    // we create almost empty objects only with objectId to reduce the amount of data transferred to the network
    List<MaterialType> materialTypes = new ArrayList<>();
    materialTypes.add( new MaterialType( Material.brake_shoes.getObjectId() ) );
    materialTypes.add( new MaterialType( Material.oil.getObjectId() ) );

    List<WorkType> workTypes = new ArrayList<>();
    workTypes.add( new WorkType( Work.brake_shoes_replacement.getObjectId() ) );
    workTypes.add( new WorkType( Work.oil_change.getObjectId() ) );
    workTypes.add( new WorkType( Work.light_calibration.getObjectId() ) );

    Auto auto = new Auto();
    auto.setBrand( "Mazda" );
    auto.setBody_type( "hatchback" );
    auto.setModel( "m3" );
    auto.setColor( "Red" );
    auto.setYear( 2013 );
    auto.setEngine_capacity( 1600 );

    ServiceOrder serviceOrder = new ServiceOrder();
    serviceOrder.setCustomer( new Person( "19386945-F71B-162A-FFB4-0EE33D4FF300" ) );
    serviceOrder.setVehicle( auto );
    serviceOrder.setMaterials( materialTypes );
    serviceOrder.setService( workTypes );
    serviceOrder.setDiscount( 0.03 );

    String serviceOrderId = Backendless.CustomService.invoke( "SaveObjectModel", "createServiceOrder", new Object[] {serviceOrder} );
    ServiceOrder savedServiceOrder = Backendless.Data.of( ServiceOrder.class ).findById( serviceOrderId );
    System.out.println("Service order: " + savedServiceOrder.getObjectId());
  }
}
